@@ -14,8 +14,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/bucoord/dump.c,v 1.12.2.3 2007/04/10 18:43:41 shadow Exp $");
 
 #include <sys/types.h>
 #include <afs/cmd.h>
@@ -178,6 +176,7 @@ bc_Dumper(aindex)
  *	free the list of volumes used for dumps
  */
 
+void
 freeDumpTaskVolumeList(vdptr)
      struct bc_volumeDump *vdptr;
 {
@@ -198,9 +197,10 @@ freeDumpTaskVolumeList(vdptr)
  *     The other half of the dump/restore create process call. In bc_StartDmpRst, 
  *     we allocated a dumpTask entry. Here we do the task and then free the entry.
  */
-bc_DmpRstStart(aindex)
-     afs_int32 aindex;
+void *
+bc_DmpRstStart(void *param)
 {
+    afs_int32 aindex = (afs_int32) param;
     register struct bc_dumpTask *tdump;
     register afs_int32 code;
 
@@ -223,7 +223,7 @@ bc_DmpRstStart(aindex)
 	free(tdump->portOffset);
     tdump->flags &= ~BC_DI_INUSE;
 
-    return code;
+    return (void *)code;
 }
 
 /* bc_StartDmpRst
