@@ -14,8 +14,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/libadmin/test/client.c,v 1.6 2003/08/08 21:54:41 shadow Exp $");
 
 #include "client.h"
 #include <afs/cellconfig.h>
@@ -38,6 +36,8 @@ RCSID
 #include <pthread.h>
 #endif
 
+/* These aren't coming from a header, currently, so they must stay here. 
+   Fix elsewhere, or leave alone. */
 extern int RXSTATS_RetrieveProcessRPCStats();
 extern int RXSTATS_RetrievePeerRPCStats();
 extern int RXSTATS_QueryProcessRPCStats();
@@ -260,7 +260,7 @@ interface_t int_list[] = {
  */
 
 int
-DoClientLocalCellGet(struct cmd_syndesc *as, char *arock)
+DoClientLocalCellGet(struct cmd_syndesc *as, void *arock)
 {
     afs_status_t st = 0;
     char cellName[MAXCELLCHARS];
@@ -275,7 +275,7 @@ DoClientLocalCellGet(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoClientMountPointCreate(struct cmd_syndesc *as, char *arock)
+DoClientMountPointCreate(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { DIRECTORY, VOLUME, READWRITE,
 	CHECK
@@ -329,7 +329,7 @@ Print_afs_serverEntry_p(afs_serverEntry_p serv, const char *prefix)
 }
 
 int
-DoClientAFSServerGet(struct cmd_syndesc *as, char *arock)
+DoClientAFSServerGet(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER } DoClientAFSServerGet_parm_t;
     afs_status_t st = 0;
@@ -350,7 +350,7 @@ DoClientAFSServerGet(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoClientAFSServerList(struct cmd_syndesc *as, char *arock)
+DoClientAFSServerList(struct cmd_syndesc *as, void *arock)
 {
     afs_status_t st = 0;
     afs_serverEntry_t entry;
@@ -474,7 +474,7 @@ GetStatTypeFromString(const char *type)
 }
 
 int
-DoClientRPCStatsStateGet(struct cmd_syndesc *as, char *arock)
+DoClientRPCStatsStateGet(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PROCESS,
 	STAT_TYPE
@@ -534,7 +534,7 @@ DoClientRPCStatsStateGet(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoClientRPCStatsStateEnable(struct cmd_syndesc *as, char *arock)
+DoClientRPCStatsStateEnable(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PROCESS, STAT_TYPE } DoClientRPCStatsEnable_parm_t;
     afs_status_t st = 0;
@@ -588,7 +588,7 @@ DoClientRPCStatsStateEnable(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoClientRPCStatsStateDisable(struct cmd_syndesc *as, char *arock)
+DoClientRPCStatsStateDisable(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PROCESS,
 	STAT_TYPE
@@ -692,7 +692,7 @@ Print_afs_RPCStats_p(afs_RPCStats_p stat, interface_function_list_p f_list,
 }
 
 int
-DoClientRPCStatsList(struct cmd_syndesc *as, char *arock)
+DoClientRPCStatsList(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PROCESS, STAT_TYPE } DoClientRPCStatsList_parm_t;
     afs_status_t st = 0;
@@ -817,7 +817,7 @@ DoClientRPCStatsList(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoClientRPCStatsClear(struct cmd_syndesc *as, char *arock)
+DoClientRPCStatsClear(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PROCESS, STAT_TYPE, CLEAR_ALL, CLEAR_INVOCATIONS,
 	CLEAR_BYTES_SENT, CLEAR_BYTES_RCVD,
@@ -989,7 +989,7 @@ DoClientRPCStatsClear(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoClientRPCStatsVersionGet(struct cmd_syndesc *as, char *arock)
+DoClientRPCStatsVersionGet(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PROCESS } DoClientRPCStatsVersionGet_parm_t;
     afs_status_t st = 0;
@@ -1045,7 +1045,7 @@ Print_afs_CMServerPref_p(afs_CMServerPref_p pref)
 }
 
 int
-DoClientCMGetServerPrefs(struct cmd_syndesc *as, char *arock)
+DoClientCMGetServerPrefs(struct cmd_syndesc *as, void *arock)
 {
     afs_status_t st = 0;
     typedef enum { SERVER, PORT } DoClientCMGetServerPrefs_parm_t;
@@ -1115,7 +1115,7 @@ Print_afs_CMListCell_p(afs_CMListCell_p cellInfo)
 }
 
 int
-DoClientCMListCells(struct cmd_syndesc *as, char *arock)
+DoClientCMListCells(struct cmd_syndesc *as, void *arock)
 {
     afs_status_t st = 0;
     typedef enum { SERVER, PORT } DoClientCMListCells_parm_t;
@@ -1170,7 +1170,7 @@ DoClientCMListCells(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoClientCMLocalCell(struct cmd_syndesc *as, char *arock)
+DoClientCMLocalCell(struct cmd_syndesc *as, void *arock)
 {
     afs_status_t st = 0;
     typedef enum { SERVER, PORT } DoClientCMLocalCell_parm_t;
@@ -1229,7 +1229,7 @@ Print_afs_ClientConfig_p(afs_ClientConfig_p config)
 }
 
 int
-DoClientCMClientConfig(struct cmd_syndesc *as, char *arock)
+DoClientCMClientConfig(struct cmd_syndesc *as, void *arock)
 {
     afs_status_t st = 0;
     typedef enum { SERVER, PORT } DoClientCMLocalCell_parm_t;
@@ -1278,12 +1278,12 @@ SetupClientAdminCmd(void)
 {
     struct cmd_syndesc *ts;
 
-    ts = cmd_CreateSyntax("ClientLocalCellGet", DoClientLocalCellGet, 0,
+    ts = cmd_CreateSyntax("ClientLocalCellGet", DoClientLocalCellGet, NULL,
 			  "get the name of this machine's cell");
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("ClientMountPointCreate", DoClientMountPointCreate,
-			  0, "create a mount point");
+			  NULL, "create a mount point");
     cmd_AddParm(ts, "-directory", CMD_SINGLE, CMD_REQUIRED,
 		"directory where mount point will be created");
     cmd_AddParm(ts, "-volume", CMD_SINGLE, CMD_REQUIRED,
@@ -1294,12 +1294,12 @@ SetupClientAdminCmd(void)
 		"check that the volume exists before mounting");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("ClientAFSServerGet", DoClientAFSServerGet, 0,
+    ts = cmd_CreateSyntax("ClientAFSServerGet", DoClientAFSServerGet, NULL,
 			  "retrieve information about an afs server");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED, "server to query");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("ClientAFSServerList", DoClientAFSServerList, 0,
+    ts = cmd_CreateSyntax("ClientAFSServerList", DoClientAFSServerList, NULL,
 			  "retrieve information about all afs "
 			  "servers in a cell");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
@@ -1312,7 +1312,7 @@ SetupClientAdminCmd(void)
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("ClientRPCStatsStateGet", DoClientRPCStatsStateGet,
-			  0, "retrieve the rpc stat collection state");
+			  NULL, "retrieve the rpc stat collection state");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where command will execute");
     cmd_AddParm(ts, "-process", CMD_SINGLE, CMD_REQUIRED,
@@ -1323,7 +1323,7 @@ SetupClientAdminCmd(void)
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("ClientRPCStatsStateEnable",
-			  DoClientRPCStatsStateEnable, 0,
+			  DoClientRPCStatsStateEnable, NULL,
 			  "set the rpc stat collection state to on");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where command will execute");
@@ -1335,7 +1335,7 @@ SetupClientAdminCmd(void)
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("ClientRPCStatsStateDisable",
-			  DoClientRPCStatsStateDisable, 0,
+			  DoClientRPCStatsStateDisable, NULL,
 			  "set the rpc stat collection state to off");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where command will execute");
@@ -1346,7 +1346,7 @@ SetupClientAdminCmd(void)
 		"stats to retrieve <peer or process>");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("ClientRPCStatsList", DoClientRPCStatsList, 0,
+    ts = cmd_CreateSyntax("ClientRPCStatsList", DoClientRPCStatsList, NULL,
 			  "list the rpc stats");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where command will execute");
@@ -1357,7 +1357,7 @@ SetupClientAdminCmd(void)
 		"stats to retrieve <peer or process>");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("ClientRPCStatsClear", DoClientRPCStatsClear, 0,
+    ts = cmd_CreateSyntax("ClientRPCStatsClear", DoClientRPCStatsClear, NULL,
 			  "reset rpc stat counters");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where command will execute");
@@ -1393,7 +1393,7 @@ SetupClientAdminCmd(void)
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("ClientRPCStatsVersionGet",
-			  DoClientRPCStatsVersionGet, 0,
+			  DoClientRPCStatsVersionGet, NULL,
 			  "list the server's rpc stats version");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where command will execute");
@@ -1403,27 +1403,27 @@ SetupClientAdminCmd(void)
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("ClientCMGetServerPrefs", DoClientCMGetServerPrefs,
-			  0, "list a client's server preferences ");
+			  NULL, "list a client's server preferences ");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where command will execute");
     cmd_AddParm(ts, "-port", CMD_SINGLE, CMD_OPTIONAL, "UDP port to query");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("ClientCMListCells", DoClientCMListCells, 0,
+    ts = cmd_CreateSyntax("ClientCMListCells", DoClientCMListCells, NULL,
 			  "list a client's CellServDB ");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where command will execute");
     cmd_AddParm(ts, "-port", CMD_SINGLE, CMD_OPTIONAL, "UDP port to query");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("ClientCMLocalCell", DoClientCMLocalCell, 0,
+    ts = cmd_CreateSyntax("ClientCMLocalCell", DoClientCMLocalCell, NULL,
 			  "get the name of the client's local cell");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where command will execute");
     cmd_AddParm(ts, "-port", CMD_SINGLE, CMD_OPTIONAL, "UDP port to query");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("ClientCMClientConfig", DoClientCMClientConfig, 0,
+    ts = cmd_CreateSyntax("ClientCMClientConfig", DoClientCMClientConfig, NULL,
 			  "get the client's cache configuration");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where command will execute");
