@@ -112,8 +112,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/ptserver/ptserver.c,v 1.21.2.14 2008/12/22 19:23:32 shadow Exp $");
 
 #include <afs/stds.h>
 #ifdef	AFS_AIX32_ENV
@@ -279,16 +277,16 @@ main(int argc, char **argv)
 	int alen;
 	lcstring(arg, argv[a], sizeof(arg));
 	alen = strlen(arg);
-	if ((strncmp(arg, "-database", alen) == 0)
-	    || (strncmp(arg, "-db", alen) == 0)) {
-	    pr_dbaseName = argv[++a];	/* specify a database */
-	} else if (strcmp(argv[a], "-d") == 0) {
+	if (strcmp(argv[a], "-d") == 0) {
 	    if ((a + 1) >= argc) {
 		fprintf(stderr, "missing argument for -d\n"); 
 		return -1; 
 	    }
 	    debuglevel = atoi(argv[++a]);
 	    LogLevel = debuglevel;
+	} else if ((strncmp(arg, "-database", alen) == 0)
+	    || (strncmp(arg, "-db", alen) == 0)) {
+	    pr_dbaseName = argv[++a];	/* specify a database */
 	} else if (strncmp(arg, "-p", alen) == 0) {
 	    lwps = atoi(argv[++a]);
 	    if (lwps > 16) {	/* maximum of 16 */
@@ -525,6 +523,10 @@ main(int argc, char **argv)
     sc[1] = 0;
     if (kerberosKeys) {
 	sc[2] = rxkad_NewServerSecurityObject(0, prdir, afsconf_GetKey, NULL);
+#ifdef USE_RXKAD_KEYTAB
+	if (rxkad_InitKeytabDecrypt(AFSDIR_SERVER_RXKAD_KEYTAB_FILEPATH) == 0)
+	    rxkad_BindKeytabDecrypt(sc[2]);
+#endif
     } else
 	sc[2] = sc[0];
 
