@@ -10,8 +10,6 @@
 #include <afsconfig.h>
 #include "afs/param.h"
 
-RCSID
-    ("$Header: /cvs/openafs/src/rx/SOLARIS/rx_knet.c,v 1.19 2004/07/28 22:34:13 shadow Exp $");
 
 #ifdef AFS_SUN5_ENV
 #include "rx/rx_kcommon.h"
@@ -36,14 +34,31 @@ RCSID
 #include "inet/ip.h"
 #include "inet/ip_if.h"
 #include "netinet/udp.h"
+#ifdef AFS_SUN510_ENV
+#include "h/ddi.h"
+#include "h/ksynch.h"
+#include "h/sunddi.h"
+#include "h/sunldi.h"
+#include "h/sockio.h"
+#include "h/cmn_err.h"
+#include "h/socket.h"
+#include "netinet/in.h"
+#endif
 
 /*
  * Function pointers for kernel socket routines
  */
+#ifdef SOLOOKUP_TAKES_SOCKPARAMS
+struct sonode *(*sockfs_socreate)
+  (struct sockparams *, int, int, int, int, int *) = NULL;
+int (*sockfs_solookup)
+  (int, int, int, struct sockparams **) = NULL;
+#else
 struct sonode *(*sockfs_socreate)
   (vnode_t *, int, int, int, int, struct sonode *, int *) = NULL;
 struct vnode *(*sockfs_solookup)
   (int, int, int, char *, int *) = NULL;
+#endif /* SOLOOKUP_TAKES_SOCKPARAMS */
 int (*sockfs_sobind)
   (struct sonode *, struct sockaddr *, int, int, int) = NULL;
 int (*sockfs_sorecvmsg)
@@ -59,6 +74,10 @@ void (*sockfs_sockfree)
   (struct sonode *);
 #endif
 
+#ifndef UDP_MOD_NAME
+#define UDP_MOD_NAME "udp"
+#endif
+
 static afs_uint32 myNetAddrs[ADDRSPERSITE];
 static int myNetMTUs[ADDRSPERSITE];
 static int numMyNetAddrs = 0;
@@ -68,14 +87,11 @@ rxi_GetIFInfo()
 {
     int i = 0;
     int different = 0;
-
+#ifndef AFS_SUN510_ENV
     ill_t *ill;
     ipif_t *ipif;
-    int rxmtu, maxmtu;
-#ifdef AFS_SUN510_ENV
-    ill_walk_context_t ctx;
 #endif
-
+    int rxmtu, maxmtu;
     int mtus[ADDRSPERSITE];
     afs_uint32 addrs[ADDRSPERSITE];
     afs_uint32 ifinaddr;
@@ -84,10 +100,58 @@ rxi_GetIFInfo()
     memset(addrs, 0, sizeof(addrs));
 
 #ifdef AFS_SUN510_ENV
-    for (ill = ILL_START_WALK_ALL(&ctx) ; ill ; ill = ill_next(&ctx, ill)) {
+    (void) rw_enter(&afsifinfo_lock, RW_READER);
+
+    for (i = 0; (afsifinfo[i].ipaddr != NULL) && (i < ADDRSPERSITE); i++) {
+
+             /* Ignore addresses which are down.. */
+            if (!(afsifinfo[i].flags & IFF_UP))
+                continue;
+
+            /* Compute the Rx interface MTU */
+	    rxmtu = (afsifinfo[i].mtu - RX_IPUDP_SIZE);
+
+	    ifinaddr = afsifinfo[i].ipaddr;
+	    if (myNetAddrs[i] != ifinaddr)
+		different++;
+
+	    /* Copy interface MTU and address; adjust maxmtu */
+	    mtus[i] = rxmtu;
+	    rxmtu = rxi_AdjustIfMTU(rxmtu);
+	    maxmtu = rxmtu * rxi_nRecvFrags +
+	        ((rxi_nRecvFrags - 1) * UDP_HDR_SIZE);
+	    maxmtu = rxi_AdjustMaxMTU(rxmtu, maxmtu);
+	    addrs[i] = ifinaddr;
+
+	    if (ifinaddr != 0x7f000001 && maxmtu > rx_maxReceiveSize) {
+		rx_maxReceiveSize = MIN(RX_MAX_PACKET_SIZE, maxmtu);
+		rx_maxReceiveSize =
+		    MIN(rx_maxReceiveSize, rx_maxReceiveSizeUser);
+	    }
+            
+    }
+    
+    (void) rw_exit(&afsifinfo_lock);
+
+    rx_maxJumboRecvSize =
+	RX_HEADER_SIZE + rxi_nDgramPackets * RX_JUMBOBUFFERSIZE +
+	(rxi_nDgramPackets - 1) * RX_JUMBOHEADERSIZE;
+    rx_maxJumboRecvSize = MAX(rx_maxJumboRecvSize, rx_maxReceiveSize);
+
+    if (different) {
+	int j;
+
+	for (j = 0; j < i; j++) {
+	    myNetMTUs[j] = mtus[j];
+	    myNetAddrs[j] = addrs[j];
+	}
+    }
+
+    return different;
+}
+
 #else
     for (ill = ill_g_head; ill; ill = ill->ill_next) {
-#endif
 #ifdef AFS_SUN58_ENV
 	/* Make sure this is an IPv4 ILL */
 	if (ill->ill_isv6)
@@ -144,17 +208,19 @@ rxi_GetIFInfo()
 
     return different;
 }
+#endif
 
 int
 rxi_FindIfMTU(afs_uint32 addr)
 {
-    ill_t *ill;
-    ipif_t *ipif;
     afs_uint32 myAddr, netMask;
     int match_value = 0;
     int mtu = -1;
 #ifdef AFS_SUN510_ENV
-    ill_walk_context_t ctx;
+    int i = 0;
+#else
+    ill_t *ill;
+    ipif_t *ipif;
 #endif
 
     if (numMyNetAddrs == 0)
@@ -171,10 +237,46 @@ rxi_FindIfMTU(afs_uint32 addr)
 	netMask = 0;
 
 #ifdef AFS_SUN510_ENV
-    for (ill = ILL_START_WALK_ALL(&ctx) ; ill ; ill = ill_next(&ctx, ill)) {
+    (void) rw_enter(&afsifinfo_lock, RW_READER);
+
+    for (i = 0; (afsifinfo[i].ipaddr != NULL) && (i < ADDRSPERSITE); i++) {
+        afs_uint32 thisAddr, subnetMask;
+    	int thisMtu;
+
+        /* Ignore addresses which are down.. */
+        if ((afsifinfo[i].flags & IFF_UP) == 0)
+            continue;
+
+        thisAddr = afsifinfo[i].ipaddr;
+        subnetMask = afsifinfo[i].netmask;
+        thisMtu = afsifinfo[i].mtu;
+
+        if ((myAddr & netMask) == (thisAddr & netMask)) {
+	   if ((myAddr & subnetMask) == (thisAddr & subnetMask)) {
+	        if (myAddr == thisAddr) {
+                    match_value = 4;
+                    mtu = thisMtu;
+                }
+
+                if (match_value < 3) {
+                    match_value = 3;
+                    mtu = thisMtu;
+                }
+           }
+
+           if (match_value < 2) {
+                match_value = 2;
+                mtu = thisMtu;
+           }
+        }
+     }
+     
+     (void) rw_exit(&afsifinfo_lock);
+
+     return mtu;
+}
 #else
     for (ill = ill_g_head; ill; ill = ill->ill_next) {
-#endif
 #ifdef AFS_SUN58_ENV
 	/* Make sure this is an IPv4 ILL */
 	if (ill->ill_isv6)
@@ -213,6 +315,7 @@ rxi_FindIfMTU(afs_uint32 addr)
 
     return mtu;
 }
+#endif
 
 /* rxi_NewSocket, rxi_FreeSocket and osi_NetSend are from the now defunct
  * afs_osinet.c. 
@@ -221,7 +324,7 @@ rxi_FindIfMTU(afs_uint32 addr)
 struct sockaddr_in rx_sockaddr;
 
 /* Allocate a new socket at specified port in network byte order. */
-struct osi_socket *
+osi_socket *
 rxk_NewSocketHost(afs_uint32 ahost, short aport)
 {
     vnode_t *accessvp;
@@ -229,12 +332,19 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     struct sockaddr_in addr;
     int error;
     int len;
+#ifdef SOLOOKUP_TAKES_SOCKPARAMS
+    struct sockparams *sp;
+#endif
 
     AFS_STATCNT(osi_NewSocket);
 
     if (sockfs_solookup == NULL) {
 	sockfs_solookup =
+#ifdef SOLOOKUP_TAKES_SOCKPARAMS
+	    (int (*)())modlookup("sockfs", "solookup");
+#else
 	    (struct vnode * (*)())modlookup("sockfs", "solookup");
+#endif
 	if (sockfs_solookup == NULL) {
 	    return NULL;
 	}
@@ -283,6 +393,14 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     }
 #endif
 
+#ifdef SOLOOKUP_TAKES_SOCKPARAMS
+    error = sockfs_solookup(AF_INET, SOCK_DGRAM, 0, &sp);
+    if (error != 0) {
+	return NULL;
+    }
+
+    so = sockfs_socreate(sp, AF_INET, SOCK_DGRAM, 0, SOV_STREAM, &error);
+#else
     accessvp = sockfs_solookup(AF_INET, SOCK_DGRAM, 0, "/dev/udp", &error);
     if (accessvp == NULL) {
 	return NULL;
@@ -290,6 +408,8 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
 
     so = sockfs_socreate(accessvp, AF_INET, SOCK_DGRAM, 0, SOV_STREAM, NULL,
 			 &error);
+#endif /* SOLOOKUP_TAKES_SOCKPARAMS */
+
     if (so == NULL) {
 	return NULL;
     }
@@ -315,17 +435,17 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
 	return NULL;
     }
 
-    return (struct osi_socket *)so;
+    return (osi_socket *)so;
 }
 
-struct osi_socket *
+osi_socket *
 rxk_NewSocket(short aport)
 {
     return rxk_NewSocketHost(htonl(INADDR_ANY), aport);
 }
 
 int
-osi_FreeSocket(register struct osi_socket *asocket)
+osi_FreeSocket(register osi_socket asocket)
 {
     extern int rxk_ListenerPid;
     struct sonode *so = (struct sonode *)asocket;
@@ -350,7 +470,11 @@ osi_FreeSocket(register struct osi_socket *asocket)
 
     /* Was sockfs_sounbind(so, 0); sockfs_sockfree(so); That's wrong */
     vp = SOTOV(so);
+ #ifdef AFS_SUN511_ENV
+    VOP_CLOSE(vp, FREAD|FWRITE, 1, (offset_t)0, CRED(), NULL);
+ #else
     VOP_CLOSE(vp, FREAD|FWRITE, 1, (offset_t)0, CRED());
+ #endif
     VN_RELE(vp);
 
     return 0;
@@ -462,6 +586,211 @@ osi_NetReceive(osi_socket so, struct sockaddr_in *addr, struct iovec *dvec,
     return error;
 }
 
+#if defined(AFS_SUN510_ENV)
+/* How often afs collects interface info. Tunable via /etc/system:      */
+/* set afs:afs_if_poll_interval = integer (value is in seconds)         */
+static int afs_if_poll_interval = 30;
+
+static timeout_id_t afs_if_poller_timeout = 0;
+
+/* Global array which holds the interface info for consumers            */
+struct afs_ifinfo afsifinfo[ADDRSPERSITE];
+
+void
+osi_StartNetIfPoller()
+{
+    (void) ddi_taskq_dispatch(afs_taskq, (void(*) (void*)) osi_NetIfPoller,
+            NULL, DDI_SLEEP);
+}
+
+void
+osi_NetIfPoller()
+{
+    cred_t *cr;
+    ldi_ident_t li = NULL;
+    ldi_handle_t lh = NULL;
+    struct lifnum lifn;
+    struct lifconf lifc;
+    struct lifreq lifr;
+    struct lifreq *lifrp;
+    struct sockaddr_in *sin4_local;
+    struct sockaddr_in *sin4_dst;
+    major_t udpmajor;
+    caddr_t lifcbuf = NULL;
+    int i, count, error, rv;
+    int ifcount;
+    int metric;
+    int index;
+    uint_t mtu;
+    uint64_t flags;
+
+    /* Get our permissions */
+    cr = CRED();
+
+    /* Initialize and open /dev/udp for receiving ioctls */
+    udpmajor = ddi_name_to_major(UDP_MOD_NAME);
+
+    error = ldi_ident_from_major(udpmajor, &li);
+    if (error) {
+        cmn_err(CE_WARN, "osi_NetIfPoller: ldi_ident_from_major failed: %d",
+            error);
+        goto cleanup;
+    }
+
+    error = ldi_open_by_name(UDP_DEV_NAME, FREAD, cr, &lh, li);
+    if (error) {
+        cmn_err(CE_WARN,
+            "osi_NetIfPoller: ldi_open_by_name failed: %d", error);
+        goto cleanup;
+    }
+
+    /* First, how many interfaces do we have? */
+    (void) bzero((void *)&lifn, sizeof(struct lifnum));
+    lifn.lifn_family   = AF_INET;
+
+    error = ldi_ioctl(lh, SIOCGLIFNUM, (intptr_t)&lifn,
+        FKIOCTL, cr, &rv);
+    if (error) {
+        cmn_err(CE_WARN,
+                "osi_NetIfPoller: ldi_ioctl: SIOCGLIFNUM failed: %d", error);
+        goto cleanup;
+    }
+    ifcount = lifn.lifn_count;
+
+    /* Set up some stuff for storing the results of SIOCGLIFCONF */
+    (void) bzero((void *)&lifc, sizeof(struct lifconf));
+
+    lifcbuf = kmem_zalloc(ifcount * sizeof(struct lifreq), KM_SLEEP);
+
+    lifc.lifc_family  = AF_INET;
+    lifc.lifc_flags   = IFF_UP;
+    lifc.lifc_len     = ifcount * sizeof(struct lifreq);
+    lifc.lifc_buf     = lifcbuf;
+
+    /* Get info on each of our available interfaces. */
+    error = ldi_ioctl(lh, SIOCGLIFCONF, (intptr_t)&lifc,
+        FKIOCTL, cr, &rv);
+    if (error) {
+        cmn_err(CE_WARN,
+                "osi_NetIfPoller: ldi_ioctl: SIOCGLIFCONF failed: %d", error);
+        goto cleanup;
+    }
+    lifrp = lifc.lifc_req;
+
+    count = 0;
+
+    /* Loop through our interfaces and pick out the info we want */
+    for (i = lifc.lifc_len / sizeof(struct lifreq);
+        i > 0; i--, lifrp++) {
+                
+        if (count >= ADDRSPERSITE)
+                break;
+
+        (void) bzero((void *)&lifr, sizeof(struct lifreq));
+
+        (void) strncpy(lifr.lifr_name, lifrp->lifr_name,
+            sizeof(lifr.lifr_name));
+
+        /* Get this interface's Flags */
+        error = ldi_ioctl(lh, SIOCGLIFFLAGS, (intptr_t)&lifr,
+            FKIOCTL, cr, &rv);
+        if (error) {
+            cmn_err(CE_WARN,
+                    "osi_NetIfPoller: ldi_ioctl: SIOCGLIFFLAGS failed: %d",
+                    error);
+            goto cleanup;
+        }
+
+        /* Ignore plumbed but down interfaces. */
+        if ((lifr.lifr_flags & IFF_UP) == 0)
+            continue;
+
+        flags = lifr.lifr_flags;
+
+        /* Get this interface's MTU */
+        error = ldi_ioctl(lh, SIOCGLIFMTU, (intptr_t)&lifr,
+            FKIOCTL, cr, &rv);
+
+        if (error) {
+            mtu = 1125;
+        } else {
+            mtu = lifr.lifr_metric;
+        }
+
+        /* Get this interface's Metric */
+        error = ldi_ioctl(lh, SIOCGLIFMETRIC, (intptr_t)&lifr,
+            FKIOCTL, cr, &rv);
+
+        if (error) {
+            metric = 0;
+        } else {
+            metric = lifr.lifr_metric;
+        }
+
+        sin4_local = (struct sockaddr_in *) &lifrp->lifr_addr;
+        sin4_dst = (struct sockaddr_in *) &lifrp->lifr_dstaddr;
+
+        /* Acquire global array write lock */
+        (void) rw_enter(&afsifinfo_lock, RW_WRITER);
+
+        /* Copy our collected data into the global array */
+        (void) strncpy(afsifinfo[count].ifname, lifrp->lifr_name,
+            sizeof(afsifinfo[count].ifname));
+        afsifinfo[count].ipaddr     = ntohl(sin4_local->sin_addr.s_addr);
+        afsifinfo[count].mtu        = mtu;
+        afsifinfo[count].netmask    = lifrp->lifr_addrlen;
+        afsifinfo[count].flags      = flags;
+        afsifinfo[count].metric     = metric;
+        afsifinfo[count].dstaddr    = ntohl(sin4_dst->sin_addr.s_addr);
+
+        /* Release global array write lock */
+        (void) rw_exit(&afsifinfo_lock);
+
+        count++;
+
+    } /* Bottom of loop: for each interface ... */
+
+  cleanup:
+    /* End of thread. Time to clean up */
+    if (lifcbuf)
+        kmem_free(lifcbuf, ifcount * sizeof(struct lifreq));
+    if (lh)
+        (void) ldi_close(lh, FREAD, cr);
+    if (li)
+        (void) ldi_ident_release(li);
+
+    if (afs_shuttingdown) {
+	/* do not schedule to run again if we're shutting down */
+	return;
+    }
+
+    /* Schedule this to run again after afs_if_poll_interval seconds */
+    afs_if_poller_timeout = timeout((void(*) (void *)) osi_StartNetIfPoller,
+        NULL, drv_usectohz((clock_t)afs_if_poll_interval * MICROSEC));
+}
+
+void
+osi_StopNetIfPoller(void)
+{
+    /* it's okay if untimeout races with StartNetIfPoller/NetIfPoller;
+     * it can handle being passed invalid ids. If StartNetIfPoller is
+     * in the middle of running, untimeout will not return until
+     * StartNetIfPoller is done */
+    untimeout(afs_if_poller_timeout);
+
+    /* if NetIfPoller is queued or running, ddi_taskq_destroy will not
+     * return until it is done */
+    ddi_taskq_destroy(afs_taskq);
+
+    rw_destroy(&afsifinfo_lock);
+
+    if (afs_termState == AFSOP_STOP_NETIF) {
+	afs_termState = AFSOP_STOP_COMPLETE;
+	osi_rxWakeup(&afs_termState);
+    }
+}
+#endif /* AFS_SUN510_ENV */
+
 void
 shutdown_rxkernel(void)
 {
@@ -509,7 +838,7 @@ rxi_GetIFInfo()
 dev_t afs_udp_rdev = (dev_t) 0;
 
 /* Allocate a new socket at specified port in network byte order. */
-struct osi_socket *
+osi_socket *
 rxk_NewSocketHost(afs_uint32 ahost, short aport)
 {
     TIUSER *udp_tiptr;
@@ -523,7 +852,7 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     afs_udp_rdev = makedevice(11 /*CLONE*/, ddi_name_to_major("udp"));
     code = t_kopen(NULL, afs_udp_rdev, FREAD | FWRITE, &udp_tiptr, CRED());
     if (code) {
-	return (struct osi_socket *)0;
+	return (osi_socket *)0;
     }
 
     code = t_kalloc(udp_tiptr, T_BIND, T_ADDR, (char **)&reqp);
@@ -534,7 +863,7 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     if (code) {
 	t_kfree(udp_tiptr, (char *)reqp, T_BIND);
 	t_kclose(udp_tiptr, 0);
-	return (struct osi_socket *)0;
+	return (osi_socket *)0;
     }
 
     reqp->addr.len = sizeof(struct sockaddr_in);
@@ -548,13 +877,13 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
 	t_kfree(udp_tiptr, (char *)reqp, T_BIND);
 	t_kfree(udp_tiptr, (char *)rspp, T_BIND);
 	t_kclose(udp_tiptr, 0);
-	return (struct osi_socket *)0;
+	return (osi_socket *)0;
     }
     if (memcmp(reqp->addr.buf, rspp->addr.buf, rspp->addr.len)) {
 	t_kfree(udp_tiptr, (char *)reqp, T_BIND);
 	t_kfree(udp_tiptr, (char *)rspp, T_BIND);
 	t_kclose(udp_tiptr, 0);
-	return (struct osi_socket *)0;
+	return (osi_socket *)0;
     }
     t_kfree(udp_tiptr, (char *)reqp, T_BIND);
     t_kfree(udp_tiptr, (char *)rspp, T_BIND);
@@ -568,17 +897,17 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     q->q_next->q_hiwat = rx_UdpBufSize;
     RD(q)->q_hiwat = rx_UdpBufSize;
 
-    return (struct osi_socket *)udp_tiptr;
+    return (osi_socket *)udp_tiptr;
 }
 
-struct osi_socket *
+osi_socket *
 rxk_NewSocket(short aport)
 {
     return rxk_NewSocketHost(htonl(INADDR_ANY), aport);
 }
 
 int
-osi_FreeSocket(register struct osi_socket *asocket)
+osi_FreeSocket(register osi_socket *asocket)
 {
     extern int rxk_ListenerPid;
     TIUSER *udp_tiptr = (TIUSER *) asocket;
@@ -681,7 +1010,7 @@ osi_NetSend(osi_socket asocket, struct sockaddr_in *addr, struct iovec *dvec,
 
 
 int
-osi_NetReceive(struct osi_socket *asocket, struct sockaddr_in *addr,
+osi_NetReceive(osi_socket *asocket, struct sockaddr_in *addr,
 	       struct iovec *dvec, int nvecs, int *alength)
 {
     int i;
