@@ -31,7 +31,7 @@ extern int fc_keysched(struct ktc_encryptionKey *key,
 extern afs_int32 fc_ecb_encrypt(void * clear, void * cipher,
 				fc_KeySchedule schedule, int encrypt);
 extern afs_int32 fc_cbc_encrypt(void *input, void *output, afs_int32 length,
-				fc_KeySchedule key, afs_uint32 * xor,
+				fc_KeySchedule key, afs_uint32 * iv,
 				int encrypt);
 
 /* rxkad_client.c */
@@ -50,14 +50,13 @@ extern int rxkad_GetResponse(struct rx_securityClass *aobj,
 extern void rxkad_ResetState(void);
 
 /* rxkad_common.c */
-#if 0
-/* can't prototype these due to types */
+struct rxkad_endpoint;
 extern int rxkad_SetupEndpoint(struct rx_connection *aconnp,
 			       struct rxkad_endpoint *aendpointp);
+struct rxkad_v2ChallengeResponse;
 extern afs_uint32 rxkad_CksumChallengeResponse(struct
 					       rxkad_v2ChallengeResponse
 					       *v2r);
-#endif
 extern int rxkad_DeriveXORInfo(struct rx_connection *aconnp,
 			       fc_KeySchedule * aschedule, char *aivec,
 			       char *aresult);
@@ -121,8 +120,12 @@ extern afs_int32 rxkad_GetServerInfo(struct rx_connection *aconn,
 				     afs_uint32 * expiration, char *name,
 				     char *instance, char *cell,
 				     afs_int32 * kvno);
-
-
+extern afs_int32 rxkad_SetConfiguration(struct rx_securityClass *aobj,
+                                        struct rx_connection *aconn, 
+                                        rx_securityConfigVariables atype,
+                                        void * avalue, void **aresult);
+extern int rxkad_SetAltDecryptProc(struct rx_securityClass *aobj,
+				   rxkad_alt_decrypt_func alt_decrypt);
 
 /* ticket.c */
 extern int tkt_DecodeTicket(char *asecret, afs_int32 ticketLen,
@@ -148,6 +151,16 @@ extern int tkt_DecodeTicket5(char *ticket, afs_int32 ticket_len,
 			     char *get_key_rock, int serv_kvno, char *name,
 			     char *inst, char *cell, char *session_key,
 			     afs_int32 * host, afs_int32 * start,
-			     afs_int32 * end);
+			     afs_int32 * end, afs_int32 disableDotCheck,
+			     rxkad_alt_decrypt_func alt_decrypt);
+/*
+ * Compute a des key from a key of a semi-arbitrary kerberos 5 enctype.
+ * Modifies keydata if enctype is 3des.
+ */
+extern int tkt_DeriveDesKey(int enctype, void *keydata, size_t keylen, struct ktc_encryptionKey
+			    *output);
+/* ticket5_keytab.c */
+extern int rxkad_InitKeytabDecrypt(const char *);
+extern int rxkad_BindKeytabDecrypt(struct rx_securityClass *);
 
 #endif
