@@ -10,7 +10,6 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/venus/gcpags.c,v 1.4 2001/07/12 19:59:27 shadow Exp $");
 
 #include <rx/xdr.h>
 #include <sys/ioctl.h>
@@ -41,17 +40,18 @@ RCSID("$Header: /cvs/openafs/src/venus/gcpags.c,v 1.4 2001/07/12 19:59:27 shadow
 #include "AFS_component_version_number.c"
 
 main(argc, argv)
-int argc;
-char **argv; {
-    afs_int32 code=0;
+     int argc;
+     char **argv;
+{
+    afs_int32 code = 0;
     struct ViceIoctl blob;
-    
+
     blob.in = 0;
     blob.out = 0;
     blob.in_size = 0;
     blob.out_size = 0;
     code = pioctl(0, VIOC_GCPAGS, &blob, 1);
-    if(code)
+    if (code)
 	perror("disable gcpags failed");
 
     exit(code);
